#!/usr/bin/perl

require "inc/init.pl";
require "inc/localize.pl";
require "inc/treeop.pl";
require "inc/util.pl";

# A parameter string is expected to extract current browsing info.
( $frlang, $thisuser, $fr, $pg, $md, $msg ) = unpack( "uS*", pack( "h*", $fields{'param'} ) );
$fr ||= "0";
$pg ||= "1";
$msg ||= "0";
if( $md eq "" ){ $md = 2; }
# $frlang	Currently used language
# $user		User id, must match that recorded for this session
# $fr		Currently accessed forum id
# $pg		Currently accessed page of forum list OR message list, depending on viewing mode
# $md		Maximal listing depth of tree of forums OR messages
# $msg		Currently accessed message id, if zero, post as top-level message; or otherwise as follow-up to this message.

# Set language. The "defined $$frlang" is to make sure the requested language is available.
if( $frlang && $definedlangs{$frlang} ){
	$lang = $frlang;
}

print "Content-type: text/html; charset=$charsets{$lang}\n";

&makedict;

$perm = &authenticate( );

&builderrmsg;
&buildlogonform;

if( $fields{'task'} ){
	( $taskname, $thisuser ) = unpack( "uS", pack( "H*", $fields{'task'} ) );
	# Double check, may be removed in the future.
	if( $thisuser != $userid ){
		&tellerror( "INVALID_SESS" );
		print $logonform;
		exit;
	}
}

if( $perm == $sExpired ){
	&tellerror( "SESS_EXPIRED" );
	print $logonform;
	exit;
}

if( !$fr || $thisuser != $userid || $perm == $sBanned || ( $perm == $sGuest && !$settings{'guestpost'} ) ){
	&tellerror( "INVALID_SESS" );
	print $logonform;
	exit;
}

# If called from the composition form, "task" field is set, so save the message.
# Each record is indexed by its id, which is set according to the "nextmsgid".
# Record format: user id, remote address, GM time (always), message title, and body. Separated by tabs.
# While title can not contain tabs, the message body can. The reader script will split the record only to 5 sections.
# The database ONLY records the content; the logically info will be recorded in the tree data file. This way the
# message can be moved freely inside the forum or between forums (topics).
# The tree data file records message title, author (user id), time, and status (close/open) information,
# So it would be redundant to record title, time, and user id in the database. However, this would make
# searching operations more efficient, since the tree data file doesn't need to be accessed.

if( $taskname eq "post" ){
	my %forumdb = ( );
	tie( %forumdb, $DB_Type, "dat/forum", &GDBM_WRCREAT, 0666 );
	$msgid = $forumdb{'nextmsgid'}++;
	$forumdb{'nummsgs'}++;
	untie( %forumdb );
	if( $thisuser ){
		&getuserinfo( $thisuser );
	} else {
		%userinfo = (
		'displayname' => $settings{'bbstitle'},
		'email' => $settings{'webmaster'},
		);
	}
	tie( %frdb, $DB_Type, "dat/fr$fr", &GDBM_WRCREAT, 0666 );
	$frdb{'nummsgs'}++;
	$fields{'title'} =~ s/<[^<>	]*>//g;
	$msgtime = time( ) - $timezone * 3600;
	$msgstatus = ( defined $fields{'noreply'} )? 0: ( ( defined $fields{'sendmail'} )? 2: 1 );
	if( $fields{'title'} =~ /^\s*$/ ){
		$fields{'title'} = $dict{'notitle'};
	}
	if( $settings{'wordfilter'} ){
		@badwords = split( /[,|\.]?\s+/, $settings{ 'badwords' . $lang } );
		if( $#badwords >= 0 ){
			foreach( @badwords ){
				$fields{'message'} =~ s/$_/"*" x length($_)/ige;
			}
		}
	}
	if( defined $fields{'appsign'} ){
		$fields{'message'} .= "\n\n" . $userinfo{'emailsign'};	# The <br> are "\n" are there to shown a line break whether or not html is stripped.
	}
	$frdb{$msgid} = join( "\t", $thisuser, $ENV{'REMOTE_ADDR'}, $msgtime, $fields{'title'}, $fields{'message'} );
	untie( %frdb );
	&readtree( "dat/fr$fr.dat" );
	my $ndatum = join( "\t", $msgid, $fields{'title'}, $userid, $msgtime, 0, $msgstatus );
	my $np = &tnode( $ndatum );
	# Append to the node $msg or the tree TOP. Note "son" and "last" are the first and last node of a bi-direction, non-circular linked list.
	my $pp = ( $msg? &searchtree( $root, $msg ): $root );
	if( !$$pp{'son'} ){
		$$pp{'son'} = $np;
		$$pp{'last'} = $np;
	} else {
		if( $settings{'appendlast'} && ( $pp != $root ) ){
			$$np{'prev'} = $$pp{'last'};
			$$pp{'last'}{'next'} = $np;
			$$pp{'last'} = $np;
		} else {
			$$np{'next'} = $$pp{'son'};
			$$pp{'son'}{'prev'} = $np;
			$$pp{'son'} = $np;
		}
	}
	$$np{'par'} = $pp;
	&writetree( "dat/fr$fr.dat" );
	if( $msg ){
		$np = &searchtree( $root, $msg );
		@msgdata = split( /\t/, $$np{'cap'}, 5 );
	}
	$root = 0;
	&readtree( "dat/topics.dat" );
	$np = &searchtree( $root, $fr );
	my @frdata = split( /\t/, $$np{'cap'}, 5);
	$frdata[2] = $msgtime;
	$frdata[3]++;
	$$np{'cap'} = join( "\t", @frdata );
	&writetree( "dat/topics.dat" );
	my $mparam = unpack( "h*", pack( "uS*", $lang, $userid, $fr, $pg, $md, $msgid ) );
	my %feedbackmsg = (
		'eng' => qq{Your message "$fields{'title'}" has been posted to the forum "$frdata[0]".},
		'fr' => qq{Votre message "$fields{'title'}" a &eacute;t&eacute; signal&eacute; au forum "$frdata[0]".},
		'de' => qq{Ihre Meldung "$fields{'title'}" ist zum Forum "$frdata[0]" bekanntgegeben worden.},
		'gb' => qq{ӡ$fields{'title'}Ѿӵ顰$frdata[0]},
		'big5' => qq{zl$fields{'title'}wg[FQײա$frdata[0]},
	);
	if( $userid ){
		&sendnews( 0, $userid, \($feedbackmsg{$lang}) );
	}
	my %emailmsg = (
		'eng' => qq{The message "$fields{'title'}" was just added to the forum "$frdata[0]" at },
		'fr' => qq{Le message "$fields{'title'}" a &eacute;t&eacute; juste ajout&eacute; au forum "$frdata[0]" &agrave; },
		'de' => qq{Die Meldung "$fields{'title'}" wurde gerade dem Forum "$frdata[0]" hinzugef&uuml;gt beim },
		'gb' => qq{ӡ$fields{'title'}ӵ顰$frdata[0]},
		'big5' => qq{sl$fields{'title'}[FQײա$frdata[0]b},
	);
	$fields{'message'} = $emailmsg{$lang} . qq{ $settings{'bbstitle'} ($settings{'siteroot'})\n} . ( '-' x 32 ) . "\n\n". $fields{'message'};
	my $from = "$userinfo{'displayname'} <$userinfo{'email'}>";
	tie( %maillist, $DB_Type, "dat/ml$fr", &GDBM_WRCREAT, 0666 );
	foreach $subscriber ( keys %maillist ){
		next if( $userinfo{'email'} eq $subscriber );
		&sendemail( $from, "Subscriber <$subscriber>", $fields{'title'}, \($fields{'message'}) );
	}
	untie( %maillist );
	if( defined $fields{'ccself'} ){
		&sendemail( $from, $from, $fields{'title'}, \($fields{'message'}) );
	}
	if( $msgdata[4] == 2 ){
		&getuserinfo( $msgdata[1] );
		$to = "$userinfo{'displayname'} <$userinfo{'email'}>";
		&sendemail( $from, $to, $fields{'title'}, \($fields{'message'}) );
	}
	if( $settings{'sendmeall'} ){
		&sendemail( $from, "Webmaster <$settings{'webmaster'}>", $fields{'title'}, \($fields{'message'}) );
	}
	print "Location: index.cgi?param=$mparam\n\n";
	exit;
}

# Now open the template file to show the composition form. The form should contain the following fields:
# <input type="Hidden" name="param" value="{%param%}">
# <input type="Hidden" name="task" value="{%newtask%}">
# where the "newtask" should be encoded value of "post"

$fields{'newtask'} = unpack( "H*", pack( "uS", "post", $thisuser ) );

if( $msg ){
	my %frdb = ( );
	tie( %frdb, $DB_Type, "dat/fr$fr", &GDBM_WRCREAT, 0666 );
	# Five fields: userid, remote address, time, title, and text
	my @msginfo = split( /\t/, $frdb{$msg}, 5 );
	untie( %frdb );
	if( $msginfo[3] =~ /^RE\[(\d+)\]:/i ){
		my $replylevel = $1 + 1;
		$fields{'title'} = "RE[$replylevel]:" . $';
	} elsif( $msginfo[3] =~ /^RE:/i ){
		$fields{'title'} = "RE[2]:" . $';
	} else {
		$fields{'title'} = "RE: " . $msginfo[3];
	}
	my @chars = split( //, $msginfo[4] );
	while( $#chars >= 0 ){
		my $p = 0;
		while( $chars[$p] =~ /\s/ && $p < $#chars ){ $p++; }
		splice( @chars, 0, $p );
		last if( $#chars < 0 );
		$p = ( $#chars > 47? 47: $#chars );
		my $t = join( "", @chars[0..$p] );
		if( $t =~ /[\n|\r]/ ){
			$t = $`;
			$p = length($t);
		} elsif( $t =~ /\s/ ){
			while( $chars[$p] =~ /\S/ && $p > 0 ){ $p--; }
		}
		$t = join( "", @chars[0..$p] );
		$t =~ s/^> //;
		$fields{'quotedtext'} .= "> $t\n";
		splice( @chars, 0, $p+1 );
	}
}

$fields{'title'} =~ s/\"/&quot;/g;
if( !$thisuser ){
	$fields{'ccselfable'} = "Disabled";
	$fields{'appsignable'} = "Disabled";
	$fields{'sendmailable'} = "Disabled";
}
if( !$settings{'autoinform'} ){
	$fields{'sendmailable'} = "Disabled";
}

print "\n";

$fields{'validatecode'} = qq{
<script language="JavaScript">
function validate( frm ){
	if( !frm.title.value.length && !frm.message.value.length ){
		alert( "$errmsg{'NO_CONTENT'}" );
		return false;
	}
	return true;
}
</script>
};

open( HTT, "<tmp/postnew.htt" );
flock( HTT, 2 );
while( <HTT> ){
	s/{%(\w+)%}/$fields{$1}/g;
	s/<%(\w+)%>/$dict{$1}/g;
	print;
}
close( HTT );
