#!/usr/bin/perl

require "inc/init.pl";
require "inc/localize.pl";
require "inc/treeop.pl";
require "inc/util.pl";

# A parameter string is expected to extract current browsing info.
( $frlang, $thisuser, $fr, $pg, $md, $msg ) = unpack( "uS*", pack( "h*", $fields{'param'} ) );

if( $frlang && $definedlangs{$frlang} ){
	$lang = $frlang;
}

print "Content-type: text/html; charset=$charsets{$lang}\n";

&makedict;

( $taskname, $thisuser ) = unpack( "uS", pack( "H*", $fields{'task'} ) );

if( $taskname eq "search" ){
	print "\n";
	open( HTT, "<tmp/search.htt" );
	flock( HTT, 2 );
	while( <HTT> ){
		s/{%(\w+)%}/$fields{$1}/g;
		s/<%(\w+)%>/$dict{$1}/g;
		print;
	}
	close( HTT );
	exit;
}

if( $taskname eq "email" ){
	&builderrmsg;
	if( !$fr || !$msg ){
		print "\n";
		&tellerror( 'FILE_NOT_EXIST' );
		exit;
	}
	my %myterms = (
	'emailer' => [ 'Email sender', 'Expditeur de courrier', 'Postabsender', 'ʼ', '' ],
	'myself' => [ 'Myself', 'Moi-mme', 'Selbst', 'Լ', '' ],
	'other' => [ 'Other', 'Autre', 'Anderes', '', '' ],
	'feedback' => ['An electronic mail has been sent to', 'Un courrier lectronique a t envoy ', 'Eine elektronische Post ist geschickt worden zum', 'ʼѾ͸', 'qllwgoe' ],
	);
	my $langsub = { 'eng' => 0, 'fr' => 1, 'de' => 2, 'gb' => 3, 'big5' => 4 }->{$lang};
	for( qw(emailer myself other) ){
		$dict{$_} = $myterms{$_}[$langsub];
	}
	if( !$fields{'initready'} ){
		&readtree( "dat/topics.dat" );
		$node = &searchtree( $root, $fr );
		if( !$node || $node == $root ){
			print "\n";
			&tellerror( 'FILE_NOT_EXIST' );
			exit;
		}
		my @frinfo = split( /\t/, $$node{'cap'} );
		my %frdb = ( );
		tie( %frdb, $DB_Type, "dat/fr$fr", &GDBM_WRCREAT, 0666 );
		my @msginfo = split( /\t/, $frdb{$msg}, 5 );
		untie( %frdb );
		$fields{'message'} ||= qq{$settings{'bbstitle'} ($settings{'siteroot'}) ==&gt; $frinfo[0] ==&gt; $msginfo[3]\n\n$msginfo[4]};
		$msginfo[3] =~ s/"/\\"/g;
		$fields{'subject'} = $msginfo[3];
		$fields{'appsignable'} = "Disabled";
		if( $thisuser ){
			&getuserinfo( $thisuser );
			$fields{'fromname'} = $userinfo{'displayname'};
			$fields{'fromemail'} = $userinfo{'email'};
			if( $userinfo{'emailsign'} ){ $fields{'appsignable'} = ""; }
		}
		%myinfo = %userinfo;
		%userinfo = 0;
		if( $msginfo[0] ){
			&getuserinfo( $msginfo[0] );
			$fields{'toname'} = $userinfo{'displayname'};
			$fields{'toemail'} = $userinfo{'email'};
		}
	} else {
		if( $fields{'fromemail'} =~ /^\S+\@(\S+\.)+\S+$/ && $fields{'toemail'} =~ /^\S+\@(\S+\.)+\S+$/ ){
			&sendemail(
			qq{$fields{'fromname'} <$fields{'fromemail'}>},
			qq{$fields{'toname'} <$fields{'toemail'}>},
			$fields{'subject'}, \( $fields{'message'} ) );
			if( $fields{'appsign'} ){
				&getuserinfo( $thisuser );
				$fields{'message'} .= "\n" . $userinfo{'emailsign'};
			}
			if( $thisuser ){
				&sendnews( 0, $thisuser, \( qq{$myterms{'feedback'}[$langsub] $fields{'toname'} &lt;$fields{'toemail'}&gt;.} ) );
			}
			print "Location: index.cgi?param=$fields{'param'}\n\n";
			exit;
		} else {
			$fields{'errmsg'} = $errmsg{'INVALID_EMAIL'};
		}
	}
	print "\n";
	$fields{'jscode'} = <<CODE;
<script language="JavaScript">
var emails = new Array( "$userinfo{'email'}", "$settings{'webmaster'}", "$myinfo{'email'}", "" );
var names = new Array( "$userinfo{'displayname'}", "$dict{'webmaster'}", "$myinfo{'displayname'}", "" );
function towhom( idx ){
	document.forms[0].toname.value = names[idx];
	document.forms[0].toemail.value = emails[idx];
}
function validate( frm ){
	if( !frm.fromemail.value.match( /^\\S+\@(\\S+\\.)+\\S+\$/ ) || !frm.toemail.value.match( /^\\S+\@(\\S+\\.)+\\S+\$/ ) ){
		alert( "$errmsg{'INVALID_EMAIL'}" );
		return false;
	}
	return true;
}
</script>
CODE
	open( HTT, "<tmp/sendmail.htt" );
	flock( HTT, 2 );
	while( <HTT> ){
		s/{%(\w+)%}/$fields{$1}/g;
		s/<%(\w+)%>/$dict{$1}/g;
		print;
	}
	close( HTT );
	exit;
}

$dep = 0;

sub printmsg{
	my $node = shift;
	my $msgid = $$node{'id'};
	# Five fields: title, id, time, reads, status
	my @ndata = split( /\t/, $$node{'cap'} );
	my @msginfo = split( /\t/, $frdb{$msgid}, 5 );
	$dep++;
	if( $$node{'id'} ){
	%thismsg = (
		'id' => $$node{'id'},
		'title' => $ndata[0],
		'authorname' => $ndata[1]? $users{$ndata[1]}: $dict{'guest'},
		'lastmod' => &customtime( $ndata[2] ),
		'reads' => $ndata[3],
		'ipaddr' => ( !$settings{'hideip'} || $mystatus{'manager'} )? $msginfo[1]: $dict{'keepsecret'},
		'text' => $msginfo[4],
	);
	push( @toc, ( "&nbsp;" x ( 4 * $dep ) ) . "# $thismsg{'id'}: $thismsg{'title'} ---- $thismsg{'authorname'}<br>\n" );
	$thismsg{'text'} =~ s{([\n|\r]+)}{ "<br>" x ( ( length($1)/2 <=> 1 ) + 1 ) }ge;
	print qq{
	<h2># $thismsg{'id'}: $thismsg{'title'}</h2>
	<b>$dict{'author'}: $thismsg{'authorname'}</b><br>
	<b>$dict{'time'}: $thismsg{'lastmod'}</b><br>
	<b>$dict{'ipaddr'}: $thismsg{'ipaddr'}</b><br>
	<hr size=1 color=black noshade>
	$thismsg{'text'}<br><br>
	};
	}
	my $ptr = $$node{'son'};
	if( $ptr ){
		while( $ptr ){
			&printmsg( $ptr );
			$ptr = $$ptr{'next'};
		}
	}
	$dep--;
}

if( $taskname eq "print" ){
	print "\n";
	&readusers;
	&readtree( "dat/topics.dat" );
	my $frnode = &searchtree( $root, $fr );
	my @frdata = split( /\t/, $$frnode{'cap'} );
	&readtree( "dat/fr$fr.dat" );
	my $np;
	if( $msg ){
		$np = &searchtree( $root, $msg );
		my @ndata = split( /\t/, $$np{'cap'} );
	} else {
		$np = $root;
	}
	print qq{
<!doctype html public "-//W3C//DTD HTML 4.0 Transitional//EN">
<html>
<head>
	<meta http-equiv="Content-type" content="text/html; charset="$fields{'charset'}">
	<title>$settings{'bbstitle'} &gt; $frdata[0]</title>
	<link rel="stylesheet" href="$baseurl/fe.css">
</head>
<body>
<h1>$settings{'bbstitle'} &gt; $frdata[0]</h1>
	};
	tie( %frdb, $DB_Type, "dat/fr$fr", &GDBM_WRCREAT, 0666 );
	&printmsg( $np );
	untie( %frdb );
	print "<h2>TOC</h2>", @toc, "</body></html>";
	exit;
}

if( $taskname eq "exportusers" ){
	&authenticate;
	if( $userid != 1 ){
		&tellerror( 'INVALID_SESS' );
		exit;
	}
	print qq{
<!doctype html public "-//W3C//DTD HTML 4.0 Transitional//EN">
<html>
<head>
	<meta http-equiv="Content-type" content="text/html; charset="$fields{'charset'}">
	<title>$settings{'bbstitle'} &gt; $frdata[0]</title>
	<link rel="stylesheet" href="$baseurl/fe.css">
</head>
<body>
<pre>
	};
	tie( %users, $DB_Type, "dat/users", &GDBM_WRCREAT, 0666 );
	for( 0..$users{'fields'}-1 ){
		$users{ 'f' . $_ } =~ /^(\w+)\t/;
		$flds{$1} = $_;
	}
	foreach( keys %users ){
		next unless( /^\d+/ );
		next if( $_ <= 1 );
		my @udata = split( /\t/, $users{$_} );
		print "$udata[$flds{'displayname'}]\t&lt;$udata[$flds{'email'}]&gt;\n";
	}
	untie( %users );
	print qq{
</pre>
</body>
</html>
	};
}
